/* Automatically generated by refactor.pl.
 *
 *
 * CMDNAME: info
 * CMDLEVEL: QCMD_AUTHED | QCMD_OPER
 * CMDARGS: 2
 * CMDDESC: Shows or changes info line.
 * CMDFUNC: csu_doinfo
 * CMDPROTO: int csu_doinfo(void *source, int cargc, char **cargv);
 * CMDHELP: Usage: INFO [<channel>] [<info line>]
 * CMDHELP: Shows or updates your current info line, which can be configured to be displayed
 * CMDHELP: when you join a channel.  Where:
 * CMDHELP: channel   - channel to set info line on.  If no channel is specified, your default
 * CMDHELP:             info line will be used.  If a channel is specified you must be known
 * CMDHELP:             (+k) on the channel.
 * CMDHELP: info line - new info line to set.  If not specified, the current info line will be
 * CMDHELP:             displayed.  If \"none\" is specified, the info line will be cleared.
 */

#include "../chanserv.h"
#include "../../lib/irc_string.h"
#include <stdio.h>
#include <string.h>

int csu_doinfo(void *source, int cargc, char **cargv) {
  nick *sender=source;
  reguser *rup=getreguserfromnick(sender);
  chanindex *cip;
  regchan *rcp;
  regchanuser *rcup;
  char linebuf[INFOLEN+10];
  char *newline="";
  int doupdate=0;

  if (cargc==0 || *cargv[0]!='#') {
    /* Global info line */
    if (cargc==1) {
      /* Setting to either one word or "none" */
      if (!ircd_strcmp(cargv[0],"none")) {
	newline="";
	doupdate=1;
      } else {
	newline=cargv[0];
	doupdate=1;
      }
    } else if (cargc>1) {
      /* More than one word: we need to stick them back together */
      snprintf(linebuf,INFOLEN,"%s %s",cargv[0],cargv[1]);
      newline=linebuf;
      doupdate=1;
    }

    if (doupdate) {
      if (rup->info)
	freesstring(rup->info);
      
      rup->info=getsstring(newline, INFOLEN);

      chanservstdmessage(sender, QM_DONE);
      csdb_updateuser(rup);
    }
    
    chanservstdmessage(sender, QM_GLOBALINFO, (rup->info && *rup->info->content)?rup->info->content:"(none)");
  } else {
    /* Channel info line */

    if (!(cip=findchanindex(cargv[0])) || !(rcp=cip->exts[chanservext]) || 
	(CIsSuspended(rcp) && !cs_privcheck(QPRIV_SUSPENDBYPASS, sender))) {
      chanservstdmessage(sender, QM_UNKNOWNCHAN, cargv[0]);
      return CMD_ERROR;
    }
    
    if ((!(rcup=findreguseronchannel(rcp, rup)) || !CUHasVoicePriv(rcup))) {
      chanservstdmessage(sender, QM_NOACCESSONCHAN, cargv[0], "info");
      return CMD_ERROR;
    }

    if (cargc>1) {
      if (rcup->info) 
	freesstring(rcup->info);

      if (!ircd_strcmp(cargv[1],"none")) 
	rcup->info=NULL;
      else	
	rcup->info=getsstring(cargv[1],INFOLEN);
      
      csdb_updatechanuser(rcup);
      chanservstdmessage(sender, QM_DONE);
    }

    chanservstdmessage(sender, QM_CHANNELINFO, cip->name->content, 
		       (rcup->info && *rcup->info->content)?rcup->info->content:"(none)");
  }

  return CMD_OK;
}
